package main

import (
	"bytes"
	"encoding/json"
	"fmt"
	"io"
	"io/ioutil"
	"net"
	"net/http"
	"os"
	"time"

	"golang.org/x/text/encoding/korean"
	"golang.org/x/text/transform"
	"gopkg.in/yaml.v2"
)

// Config 구조체 정의
type Config struct {
	ServerAddress     string `yaml:"server_address"`
	MaxRetries        int    `yaml:"max_retries"`
	ReconnectInterval int    `yaml:"reconnect_interval"`
	ReadTimeout       int    `yaml:"read_timeout"`
	RestAPIEndpoint   string `yaml:"rest_api_endpoint"`
}

// LogEntry 구조체 정의 (이전과 동일)
type LogEntry struct {
	Size     byte
	Type     uint8
	DateTime string
	On       uint8
	Address  [7]byte
	Area     [24]byte
	Device   [24]byte
}

// JSONLogEntry 구조체 정의 (이전과 동일)
type JSONLogEntry struct {
	CallNo   string `json:"callNo"`
	Type     uint8  `json:"type"`
	DateTime string `json:"dateTime"`
	Status   string `json:"status"`
	Address  string `json:"addr"`
	Area     string `json:"zoneNm"`
	Device   string `json:"deviceNm"`
}

// callNo	string
// type	int
// status	string
// zoneNm	string
// deviceNm	string
// addr	string

var config Config

func loadConfig() error {
	data, err := ioutil.ReadFile("config.yaml")
	if err != nil {
		return err
	}

	return yaml.Unmarshal(data, &config)
}

func main() {
	if err := loadConfig(); err != nil {
		fmt.Printf("Error loading config: %v\n", err)
		os.Exit(1)
	}

	fmt.Print(config)

	for {
		if err := runClient(); err != nil {
			fmt.Printf("Error: %v\n", err)
			time.Sleep(time.Duration(config.ReconnectInterval) * time.Second)
		}
	}
}

func runClient() error {
	var conn net.Conn
	var err error

	for i := 0; i < config.MaxRetries; i++ {
		conn, err = net.Dial("tcp", config.ServerAddress)
		if err == nil {
			break
		}
		fmt.Printf("Connection attempt %d failed: %v\n", i+1, err)
		time.Sleep(time.Duration(config.ReconnectInterval) * time.Second)
	}

	if err != nil {
		return fmt.Errorf("failed to connect after %d attempts", config.MaxRetries)
	}
	defer conn.Close()

	fmt.Println("Connected to server")

	for {
		if err := handleConnection(conn); err != nil {
			return err
		}
	}
}

func handleConnection(conn net.Conn) error {
	for {
		// 읽기 타임아웃 설정
		conn.SetReadDeadline(time.Now().Add(time.Duration(config.ReadTimeout) * time.Second))

		// 첫 번째 바이트 (데이터 길이) 읽기
		sizeBuf := make([]byte, 1)
		_, err := conn.Read(sizeBuf)
		if err != nil {
			return fmt.Errorf("error reading size: %v", err)
		}

		size := int(sizeBuf[0])

		// size가 0인 경우 처리
		if size == 0 {
			// fmt.Println("Received packet with size 0, skipping...")
			continue
		}

		fmt.Printf("Received size: %d\n", size)
		// 나머지 데이터 읽기
		buf := make([]byte, size-1)
		_, err = io.ReadFull(conn, buf)
		if err != nil {
			return fmt.Errorf("error reading data: %v", err)
		}

		fmt.Printf("Received data: %v\n", buf)
		fmt.Printf("Received size: %v\n", len(buf))
		// 전체 데이터 (크기 바이트 포함)
		fullData := append(sizeBuf, buf...)

		logEntry, err := Deserialize(fullData)
		if err != nil {
			return fmt.Errorf("deserialization error: %v", err)
		}

		jsonData, err := convertToJSON(logEntry)
		if err != nil {
			return fmt.Errorf("JSON conversion error: %v", err)
		}

		fmt.Printf("JSON data: %s\n", jsonData)

		if err := sendToRESTAPI(jsonData); err != nil {
			return fmt.Errorf("REST API error: %v", err)
		}
	}
}

func Deserialize(data []byte) (*LogEntry, error) {
	// if len(data) != 64 {
	// 	return nil, fmt.Errorf("invalid data length: expected 64, got %d", len(data))
	// }

	fmt.Printf("Data: %v\n", data)

	le := &LogEntry{}
	le.Size = data[0]
	if le.Size == 0 {
		return nil, fmt.Errorf("invalid data length: expected , got %d", le.Size)
	}

	// 수신기에서 넘어오는 타입 값
	// 1: 화재
	// 2: 고장
	// 3: 설비
	// 4: 출력
	// 5: 운영
	// 6: 축적
	// 7: 아나로그감지기 예비경보
	// 8: 화재복구
	le.Type = data[1]
	le.DateTime = fmt.Sprintf("%02d-%02d-%02d %02d:%02d:%02d", (int(data[2]) + 2000), data[3], data[4], data[5], data[6], data[7])
	le.On = data[8]
	copy(le.Address[:], data[9:16])
	copy(le.Area[:], data[16:40])
	copy(le.Device[:], data[40:])

	// 로그 출력
	fmt.Printf(" Deserialized Packet:\n")
	fmt.Printf(" Log_size: %d\n", le.Size)
	fmt.Printf(" Log_type: %d\n", le.Type)
	fmt.Printf(" Log_datetime: %s\n", le.DateTime)
	fmt.Printf(" Log_On: %d\n", le.On)
	fmt.Printf(" Log_Address: %d%d-%d-%d%d%d-%d\n", le.Address[0], le.Address[1], le.Address[2], le.Address[3], le.Address[4], le.Address[5], le.Address[6])

	areaStr, _ := decodeEUCKR(bytes.TrimRight(le.Area[:], "\x00"))
	fmt.Printf(" Log_Area: %s\n", areaStr)

	//50 195 254 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
	deviceStr, _ := decodeEUCKR(bytes.TrimRight(le.Device[:], "\x00"))
	fmt.Printf(" Log_Device: %s\n", deviceStr)

	//191 161 186 241 0 132 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0
	return le, nil
}

func convertToJSON(le *LogEntry) ([]byte, error) {
	area, err := decodeEUCKR(bytes.TrimRight(le.Area[:], "\x00"))
	if err != nil {
		return nil, err
	}

	fmt.Printf("%d\n", len(le.Device[:]))
	device, err := decodeEUCKR(bytes.TrimRight(le.Device[:], "\x00"))
	if err != nil {
		return nil, err
	}

	// 로그 타입 변경
	switch le.Type {
	case 1:
		le.Type = 6 // 화재
	case 2:
		le.Type = 2 // 고장
	case 3:
		le.Type = 3 // 설비
	case 4:
		le.Type = 4 // 출력
	}

	jsonLE := JSONLogEntry{
		CallNo:   "010-0000-0000",
		Type:     le.Type,
		DateTime: le.DateTime,
		Status: func() string {
			if le.On == 1 {
				return "동작"
			}
			return "해제"
		}(),
		Address: fmt.Sprintf("%d%d-%d-%d%d%d-%d",
			le.Address[0], le.Address[1], le.Address[2],
			le.Address[3], le.Address[4], le.Address[5], le.Address[6]),
		Area:   area,
		Device: device,
	}

	return json.Marshal(jsonLE)
}

func decodeEUCKR(b []byte) (string, error) {
	decoder := korean.EUCKR.NewDecoder()
	result, _, err := transform.Bytes(decoder, b)
	if err != nil {
		return "", err
	}
	return string(result), nil
}

func sendToRESTAPI(data []byte) error {
	resp, err := http.Post(config.RestAPIEndpoint, "application/json", bytes.NewBuffer(data))
	if err != nil {
		return err
	}
	defer resp.Body.Close()

	if resp.StatusCode != http.StatusOK {
		return fmt.Errorf("unexpected status code: %d", resp.StatusCode)
	}

	return nil
}
